# -*- coding: utf-8 -*-
"""
/***************************************************************************
 KomunalnaInfrastruktura
                                 A QGIS plugin
 Ovaj program nudi varijante komunalnih prikljucaka
                              -------------------
        begin                : 2018-03-22
        git sha              : $Format:%H$
        copyright            : (C) 2018 by Geodezija d.o.o.
        email                : marko.sljivaric@geodezija.hr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt5.QtCore import QSettings, QTranslator, qVersion, QCoreApplication, Qt
from PyQt5.QtWidgets import QAction, QMessageBox#, QIcon
from qgis.core import *
from qgis.gui import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from qgis.utils import Qgis as Qgis
import qgis.utils
import psycopg2
from osgeo import ogr
# Initialize Qt resources from file resources.py
from .resources import *

# Import the code for the DockWidget
from .komunalna_infrastruktura_dockwidget import KomunalnaInfrastrukturaDockWidget
import os.path
import math
from qgis.networkanalysis import *

class KomunalnaInfrastruktura:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface

        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)

        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'KomunalnaInfrastruktura_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&Komunalna infrastruktura')
        # TODO: We are going to let the user set this up in a future iteration
        self.toolbar = self.iface.addToolBar(u'KomunalnaInfrastruktura')
        self.toolbar.setObjectName(u'KomunalnaInfrastruktura')

        #print "** INITIALIZING KomunalnaInfrastruktura"

        self.pluginIsActive = False
        self.dockwidget = None
        self.dockwidget = KomunalnaInfrastrukturaDockWidget()
        self.pika = QgsPointXY()
        self.piknuto = False
#         self.mouse = None
        conn_string = "host='localhost' dbname='gispodi' user='postgres' password='podigis'"
        conn = psycopg2.connect(conn_string)
        self.cursor = conn.cursor()
        self.kanalizacija = None
        self.vodovod = None
        self.struja = None
        self.telefon = None
        self.deoleinizacija = None
        self.indeksi = [ [],[],[],[],[] ]
        self.troskovi = [ [],[],[],[],[] ]
        self.putanje = [ [],[],[],[],[] ]
        self.privat = [ [],[],[],[],[] ]
        self.ciljevi = [ QgsPointXY(), QgsPointXY(), QgsPointXY(), QgsPointXY(), QgsPointXY()]
        return
        
    def deactivateTool(self):
#         QgsMessageLog.logMessage("DEAKTIVACIJA")
        self.iface.actionPan().trigger()        
    # noinspection PyMethodMayBeStatic
        return

    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('KomunalnaInfrastruktura', message)

    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.toolbar.addAction(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/KomunalnaInfrastruktura/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'Komunalna infrastruktura'),
            callback=self.run,
            parent=self.iface.mainWindow())
#         self.mouse = PointTool(self.iface.mapCanvas(), self.dockwidget, self)
        return

    def onClosePlugin(self):
        """Cleanup necessary items here when plugin dockwidget is closed"""

        #print "** CLOSING KomunalnaInfrastruktura"

        # disconnects
        self.dockwidget.closingPlugin.disconnect(self.onClosePlugin)

        # remove this statement if dockwidget is to remain
        # for reuse if plugin is reopened
        # Commented next statement since it causes QGIS crashe
        # when closing the docked window:
        # self.dockwidget = None
        self.brisiRadneSlojeve()
        self.pluginIsActive = False
#         self.mouse.deactivate()
        self.pika = None
        self.piknuto = False
        self.pluginIsActive = False
        return

    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""

        #print "** UNLOAD KomunalnaInfrastruktura"

        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&Komunalna infrastruktura'),
                action)
            self.iface.removeToolBarIcon(action)
        # remove the toolbar
        del self.toolbar
        return

    #--------------------------------------------------------------------------

    def run(self):
        """Run method that loads and starts the plugin"""
        if not self.pluginIsActive:
            self.pluginIsActive = True
            if self.dockwidget == None:
                # Create the dockwidget (after translation) and keep reference
                self.dockwidget = KomunalnaInfrastrukturaDockWidget()
            conn_string = "host='localhost' dbname='gispodi' user='postgres' password='podigis'"
            conn = psycopg2.connect(conn_string)
            self.cursor = conn.cursor()
            # connect to provide cleanup on closing of dockwidget
            self.dockwidget.closingPlugin.connect(self.onClosePlugin)
            self.dockwidget.gumb.clicked.connect(self.zapamti)
            self.dockwidget.gumb.setEnabled(False)
            self.dockwidget.cboKANALIZACIJA.addItem('Nema rjesenja')
            self.dockwidget.cboDEOLEINIZACIJA.addItem('Nema rjesenja')
            self.dockwidget.cboVODOVOD.addItem('Nema rjesenja')
            self.dockwidget.cboSTRUJA.addItem('Nema rjesenja')
            self.dockwidget.cboTELEFON.addItem('Nema rjesenja')
            self.dockwidget.cboDEOLEINIZACIJA.currentIndexChanged.connect(self.cboDEOLEINIZACIJAchange)
            self.dockwidget.cboKANALIZACIJA.currentIndexChanged.connect(self.cboKANALIZACIJAchange)
            self.dockwidget.cboSTRUJA.currentIndexChanged.connect(self.cboSTRUJAchange)
            self.dockwidget.cboTELEFON.currentIndexChanged.connect(self.cboTELEFONchange)
            self.dockwidget.cboVODOVOD.currentIndexChanged.connect(self.cboVODOVODchange)
            self.dockwidget.labK.setText("")
            self.dockwidget.labD.setText("")
            self.dockwidget.labV.setText("")
            self.dockwidget.labT.setText("")
            self.dockwidget.labS.setText("")
            self.iface.addDockWidget(Qt.RightDockWidgetArea, self.dockwidget)
            crsx = self.cursor
            sqlx = "SELECT EXISTS(SELECT 1 FROM information_schema.tables WHERE table_catalog = 'gispodi' AND table_schema = 'gis_podi' AND table_name = 'radna_parcela'); "
            crsx.execute(sqlx)
            if bool(crsx.fetchone()[0]):
                sqlx = "SELECT * FROM gis_podi.radna_parcela;"
                crsx.execute(sqlx)
                if crsx.rowcount > 0:
                    self.crtajPoligon("PrivremenaParcela")
                    self.dockwidget.show()
                    self.inicijalizacija()
                else:
                    self.iface.messageBar().pushMessage("U bazi ne postoji radna(privremena) parcela..", " Treba je prvo nacrtati..", level=QgsMessageBar.WARNING, duration=3)
                    self.onClosePlugin() 
            else:
                self.iface.messageBar().pushMessage("U bazi ne postoji radna(privremena) parcela..", " Treba je prvo nacrtati..", level=QgsMessageBar.WARNING, duration=3)
                self.onClosePlugin()
        return

    def zapamti(self):
        # 1. rijesiti PrivremenuParcelu u QGISu
        # 2. rijesiti tablice javne_infrastrukture i cilja u PostGISu
        # 3. zatvoriti program
        # Ad 1:
        self.crtajPoligon("PrivremenaParcela")
        # Ad 2:
        if self.ciljevi[0] != None:
            self.izradaJavneInfrastruktureUBaziPodataka(self.kanalizacija, "kanalizacija", self.ciljevi[0])
        if self.ciljevi[1] != None:
            self.izradaJavneInfrastruktureUBaziPodataka(self.vodovod, "voda", self.ciljevi[1])
        if self.ciljevi[2] != None:
            self.izradaJavneInfrastruktureUBaziPodataka(self.struja, "struja", self.ciljevi[2])
        if self.ciljevi[3] != None:
            self.izradaJavneInfrastruktureUBaziPodataka(self.telefon, "telefon", self.ciljevi[3])
        if self.ciljevi[4] != None:
            self.izradaJavneInfrastruktureUBaziPodataka(self.deoleinizacija, "deoleinizacija", self.ciljevi[4])
        # Ad 3:
        self.iface.messageBar().pushMessage("Komunalna infrastruktura", "Baza je dopunjena...", level=QgsMessageBar.INFO, duration=3)
        self.onClosePlugin()
        return
    
    def pronadjiSloj(self, imeSloja):
        sloj = None
        for lyr in QgsProject.instance().mapLayers().values():
            if lyr.name() == imeSloja:
                sloj = lyr
#                 QgsMessageLog.logMessage("Pronadjen layer..")
                break
        if sloj.featureCount() == 0:
            sloj = None
        return sloj
    
    def inicijalizacija(self):
        QgsMessageLog.logMessage("___________________________________")
#         tocka = self.pika
#         QgsMessageLog.logMessage("Odabrane koordinate >> " + str(tocka.x()) + ";" + str(tocka.y()))
        crsx = self.cursor
        cesticaJePronadjena = False
        sloj = self.pronadjiSloj("PrivremenaParcela")            
        if sloj != None:    
            for ft in sloj.getFeatures():
                tocke_cestice = self.extractPoints(ft.geometry())
                tekst = "'LINESTRING("
                maxIndex = len(tocke_cestice) - 1
                indeks = -1
                for t in tocke_cestice:
                    indeks = indeks + 1
                    e = t[0].x()
                    n = t[0].y()
                    tekst = tekst + str(e) + " " + str(n)
                    if indeks < maxIndex:
                        tekst = tekst + ","
                    else:
                        tekst = tekst +")', 3765"
                    
                sqlx = "DROP TABLE IF EXISTS inicijalno;"
                crsx.execute(sqlx)
                sqlx = "CREATE TEMPORARY TABLE inicijalno AS SELECT ST_MakePolygon(ST_GeomFromText(" + tekst + ")) as geom;"
                crsx.execute(sqlx)
                sqlx = "BEGIN;"
                crsx.execute(sqlx)
                sqlx = "DROP TABLE IF EXISTS gis_podi.radna_parcela;"
                crsx.execute(sqlx)
                sqlx = "CREATE TABLE gis_podi.radna_parcela AS SELECT ST_SetSRID(x.geom, 3765) AS geom FROM inicijalno AS x;"
                crsx.execute(sqlx)
                sqlx = "COMMIT;"
                crsx.execute(sqlx)
            QgsMessageLog.logMessage("Inicijalizacija...")
            self.infrastruktura()
            
        return
             
    def kreiraj_memory_layer(self, znacajke, naziv_lejera, tip_geometrije):
#         QgsMessageLog.logMessage("Kreira se lejer: " + naziv_lejera)
        korijen = QgsProject.instance().layerTreeRoot()
        grupa = korijen.findGroup("PrivremeneParcele")
        vL = QgsVectorLayer(tip_geometrije, naziv_lejera, "memory")
        pr = vL.dataProvider()
        n = 0
        vL.startEditing()
        for znacajka in znacajke:
            n = n + 1
            pr.addAttributes([QgsField("id", QVariant.Int)])
            znacajka.setAttributes([n])
            pr.addFeatures([znacajka])
        vL.commitChanges()
        vL.updateExtents()
        QgsProject.instance().addMapLayer(vL, False)
        if grupa is None:
            grupa = korijen.addGroup("PrivremenaParcela")
        grupa.addLayer(vL)
        return
        
    def crtaj_u_layer(self, znacajke, naziv_lejera, tip_geometrije):
        lejerVecPostoji = False
        korijen = QgsProject.instance().layerTreeRoot()
        grupa = korijen.findGroup("PrivremeneParcele")
        sloj = None
        for lyr in QgsProject.instance().mapLayers().values():
            if lyr.name() == naziv_lejera:
                sloj = lyr
                lejerVecPostoji = True
                break
        if sloj == None:
            sloj = QgsVectorLayer(tip_geometrije, naziv_lejera, "memory")
        pr = sloj.dataProvider()
        n = 0
        sloj.startEditing()
        for znacajka in znacajke:
            n = n + 1
            if "_infrastruktura" in naziv_lejera:
                duzina = znacajka.geometry().length()
                index = sloj.fieldNameIndex('duzina')
                if index == -1:
                    pr.addAttributes([QgsField( 'duzina', QVariant.Double)])
                    sloj.updateFields()
                znacajka.setAttributes([ duzina ])
            else:
                pr.addAttributes([QgsField("id", QVariant.Int)])
                znacajka.setAttributes([n])
            pr.addFeatures([znacajka])
        sloj.commitChanges()
        sloj.updateExtents()
        if lejerVecPostoji == False:
            QgsProject.instance().addMapLayer(sloj, False)
            if grupa is None:
                grupa = korijen.addGroup("PrivremeneParcele")
            grupa.addLayer(sloj)
        return

    def extractPoints( self, geom):
        izlaz = []
        for v in geom.vertices():
            izlaz.append(v)
        return izlaz

    def smjerniKut(self, qA, qB):
        # smjerni kut od toke A k toki B
        # qA i qB su reprezentacije toaka A i B u QgsPoint() formatu
        dE = qB.x() - qA.x()
        dN = qB.y() - qA.y()
        arctg = math.atan2(dE,dN)
#         QgsMessageLog.logMessage(str(arctg*180/math.pi))
        if dE >= 0 and dN >= 0:
            return self.limb(arctg)
        if dE >= 0 and dN < 0:
            return self.limb(arctg)
        if dE < 0 and dN < 0:
            return self.limb(arctg)
        if dE < 0 and dN >= 0:
            return self.limb(arctg)
    
    def limb(self, x):
        # funkcija svodi pravac u interval [0,360> u radijanima
        while x >= (2 * math.pi):
            x = x - 2 * math.pi
        while x < 0:
            x = x + 2 * math.pi
        return x
    
    def crtajPoligon(self, imeTablice):
        imeTablicePG = imeTablice
        precrtavanje = False
        if imeTablice == "PrivremenaParcela":
            imeTablicePG = "gis_podi.radna_parcela"
            precrtavanje = True
        crsx = self.cursor
        sqlx = "SELECT ST_AsGeoJSON(x.geom) FROM " + imeTablicePG + " AS x;"
        crsx.execute(sqlx)
        imena_stupaca = [x[0] for x in crsx.description]
        trazeni_indeks = -1
        i = -1
        for stupac in imena_stupaca:
            i = i + 1
            if stupac == "geom":
                trazeni_indeks = i
        fts = []
#         QgsMessageLog.logMessage(imeTablice + ":")
        if crsx.rowcount > 0:
#             QgsMessageLog.logMessage("Broj redaka >> " + str(crsx.rowcount))
            for redak in crsx.fetchall():
#                 QgsMessageLog.logMessage("Redak >> " + str(redak))
                if redak[0] != None:
                    js = redak[trazeni_indeks]
        #             QgsMessageLog.logMessage(str(js))
                    g = ogr.CreateGeometryFromJson(js)
                    q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
                    ft = QgsFeature()
                    ft.setGeometry(q)
                    fts.append(ft)
#             self.napraviMemorijskiSloj(fts, imeTablice, "Polygon?crs=EPSG:3765")
            if precrtavanje:
                self.precrtajSloj(fts, imeTablice, "Polygon?crs=EPSG:3765")
            else:
                self.crtajUSloj(fts, imeTablice, "Polygon?crs=EPSG:3765")
        return
    
    def crtajUSloj(self, znacajke, naziv_lejera, tip_geometrije):
        lejerVecPostoji = False
        korijen = QgsProject.instance().layerTreeRoot()
        grupa = korijen.findGroup("PrivremeneParcele")
        sloj = None
        for lyr in QgsProject.instance().mapLayers().values():
            if lyr.name() == naziv_lejera:
                sloj = lyr
                lejerVecPostoji = True
                break
        if sloj == None:
            sloj = QgsVectorLayer(tip_geometrije, naziv_lejera, "memory")
        pr = sloj.dataProvider()
        n = 0
        sloj.startEditing()
        for znacajka in znacajke:
            n = n + 1
            pr.addAttributes([QgsField("id", QVariant.Int)])
            znacajka.setAttributes([n])
            pr.addFeatures([znacajka])
        sloj.commitChanges()
        sloj.updateExtents()
        if lejerVecPostoji == False:
            QgsProject.instance().addMapLayer(sloj, False)
            if grupa is None:
                grupa = korijen.addGroup("PrivremeneParcele")
            grupa.addLayer(sloj)
        return
    
    def nulirajSloj(self, naziv_lejera, tip_geometrije):
        lejerVecPostoji = False
        korijen = QgsProject.instance().layerTreeRoot()
        grupa = korijen.findGroup("PrivremeneParcele")
        sloj = None
        for lyr in QgsProject.instance().mapLayers().values():
            if lyr.name() == naziv_lejera:
                sloj = lyr
                lejerVecPostoji = True
                break
        if sloj == None:
            sloj = QgsVectorLayer(tip_geometrije, naziv_lejera, "memory")
        pr = sloj.dataProvider()
        sloj.startEditing()
        if lejerVecPostoji:
            for znacajka in sloj.getFeatures():
                sloj.changeAttributeValue(znacajka.id(), 0, -1.0)
        else:
            pr.addAttributes([QgsField( "duzina", QVariant.Double)])
            sloj.updateFields()
            znacajka = QgsFeature()
            znacajka.setAttributes( [-1.0] )
            pr.addFeatures([znacajka])
        sloj.commitChanges()
        sloj.updateExtents()
        if lejerVecPostoji == False:
            QgsProject.instance().addMapLayer(sloj, False)
            if grupa is None:
                grupa = korijen.addGroup("PrivremeneParcele")
            grupa.addLayer(sloj)
        return
                  
    def precrtajSloj(self, znacajke, naziv_lejera, tip_geometrije):
        lejerVecPostoji = False
        korijen = QgsProject.instance().layerTreeRoot()
        grupa = korijen.findGroup("PrivremeneParcele")
        sloj = None
        for lyr in QgsProject.instance().mapLayers().values():
            if lyr.name() == naziv_lejera:
                sloj = lyr
                lejerVecPostoji = True
                break
        if sloj == None:
            sloj = QgsVectorLayer(tip_geometrije, naziv_lejera, "memory")
        pr = sloj.dataProvider()
        n = 0
        if lejerVecPostoji:
            with edit(sloj):
                listOfIds = [feat.id() for feat in sloj.getFeatures()]
                sloj.deleteFeatures( listOfIds )
        sloj.startEditing()
        for znacajka in znacajke:
            if "_infrastruktura" in naziv_lejera:
                duzina = znacajka.geometry().length()
                index = sloj.fieldNameIndex('duzina')
                if index == -1:
                    pr.addAttributes([QgsField( "duzina", QVariant.Double)])
                    sloj.updateFields()
                    index = 0
                znacajka.setAttributes([duzina])
            else:
                n = n + 1
                pr.addAttributes([QgsField("id", QVariant.Int)])
                znacajka.setAttributes([n])
            pr.addFeatures([znacajka])
        sloj.commitChanges()
        sloj.updateExtents()
        if "_infrastruktura" in naziv_lejera:
            if "kanalizacija" in naziv_lejera:
                sloj.loadNamedStyle(self.plugin_dir +'\Kanalizacija.qml')
            elif "voda" in naziv_lejera:
                sloj.loadNamedStyle(self.plugin_dir +'\Voda.qml')
            elif "deoleinizacija" in naziv_lejera:
                sloj.loadNamedStyle(self.plugin_dir +'\Deoleinizacija.qml')
            elif "telefon" in naziv_lejera:
                sloj.loadNamedStyle(self.plugin_dir +'\Telefon.qml')
            elif "struja" in naziv_lejera:
                sloj.loadNamedStyle(self.plugin_dir +'\Struja.qml')
        if lejerVecPostoji == False:
            QgsProject.instance().addMapLayer(sloj, False)
            if grupa is None:
                grupa = korijen.addGroup("PrivremeneParcele")
            grupa.addLayer(sloj)
        return        

    def zaokruziNaDvijeDecimale(self, x):
        if x == None:
            return None
        else:
            return round(int(math.ceil(x / 0.05)) * 0.05, 2)
                                
    def cboDEOLEINIZACIJAchange(self,i):
        self.deoleinizacija = self.putanje[4][i]
        self.precrtavanjeJavneInfrastrukture(4,i)
        return
        
    def cboKANALIZACIJAchange(self,i):
        self.kanalizacija = self.putanje[0][i]
        self.precrtavanjeJavneInfrastrukture(0,i)
        return
    
    def cboSTRUJAchange(self,i):
        self.struja = self.putanje[2][i]
        self.precrtavanjeJavneInfrastrukture(2,i)
        return
    
    def cboTELEFONchange(self,i):
        self.telefon = self.putanje[3][i]
        self.precrtavanjeJavneInfrastrukture(3,i)
        return
    
    def cboVODOVODchange(self,i):
        self.vodovod = self.putanje[1][i]
        self.precrtavanjeJavneInfrastrukture(1,i)
        return
    
    def precrtavanjeJavneInfrastrukture(self, indeksInfrastrukture, indeksGrane):
        prefiks = ""
        grana = []
        trosak = self.troskovi[indeksInfrastrukture][indeksGrane]
        sDuljinaJavne = "{0:.2f}".format(float(trosak))
        sDuljinaPrivatne = "{0:.2f}".format(self.privat[indeksInfrastrukture])
        sDuljinaUkupno = "{0:.2f}".format(float(trosak) + self.privat[indeksInfrastrukture])
        recenica = sDuljinaPrivatne + " [m] + " + sDuljinaJavne + " [m] = " + sDuljinaUkupno + " [m]"
        if indeksInfrastrukture == 0:
            prefiks = "kanalizacija"
            grana = self.kanalizacija
            self.dockwidget.labK.setText(recenica)
            self.dockwidget.labK.repaint()
        elif indeksInfrastrukture == 1:
            prefiks = "voda"
            grana = self.vodovod
            self.dockwidget.labV.setText(recenica)
            self.dockwidget.labV.repaint()
        elif indeksInfrastrukture == 2:
            prefiks = "struja"
            grana = self.struja
            self.dockwidget.labS.setText(recenica)
            self.dockwidget.labS.repaint()
        elif indeksInfrastrukture == 3:
            prefiks = "telefon"
            grana = self.telefon
            self.dockwidget.labT.setText(recenica)
            self.dockwidget.labT.repaint()
        elif indeksInfrastrukture == 4:
            prefiks = "deoleinizacija"
            grana = self.deoleinizacija
            self.dockwidget.labD.setText(recenica)
            self.dockwidget.labD.repaint()
        pline = []
        for pt in grana:
            qPT = QgsPointXY(pt.x(), pt.y())
            pline.append(qPT)
        ft = QgsFeature()
        ft.setGeometry(QgsGeometry.fromPolylineXY(pline))
        znacajke = []
        znacajke.append(ft)
        self.precrtajSloj(znacajke, prefiks + "_javna_infrastruktura", "LineString?crs=EPSG:3765")
        cilj = QgsPointXY(grana[0], grana[0])
        self.ciljevi[indeksInfrastrukture] = cilj
        ft = QgsFeature()
        ft.setGeometry(QgsGeometry.fromPointXY(cilj))
        znacajke = []
        znacajke.append(ft)
        self.precrtajSloj(znacajke, prefiks + "_cilj", "Point?crs=EPSG:3765")
        self.izradaJavneInfrastruktureUBaziPodataka(grana, prefiks, cilj)
        return
        
    def izradaJavneInfrastruktureUBaziPodataka(self, zapamcenaPline, ikp, zadnjaTocka):
        crsx = self.cursor
        polyLine2 = QgsGeometry.fromPolylineXY(zapamcenaPline)
        tocke = self.extractPoints(polyLine2)
        tekst = "'LINESTRING("
        maxIndex = len(tocke) - 1
        indeks = -1
        for t in tocke:
            indeks = indeks + 1
            e = t.x()
            n = t.y()
            tekst = tekst + str(e) + " " + str(n)
            if indeks < maxIndex:
                tekst = tekst + ","
            else:
                tekst = tekst +")', 3765"
        sqlx = "DROP TABLE IF EXISTS privremena_javna_infrastruktura;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE privremena_javna_infrastruktura AS SELECT ST_GeomFromText(" + tekst + ") as geom;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS gis_podi." + ikp + "_javna_infrastruktura;"
        crsx.execute(sqlx)
        sqlx = "BEGIN;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE gis_podi." + ikp + "_javna_infrastruktura(gid serial primary key NOT NULL);"
        crsx.execute(sqlx)
        sqlx = "SELECT AddGeometryColumn ('gis_podi', '" + ikp + "_javna_infrastruktura', 'geom', 3765, 'LINESTRING', 2, false);"
        crsx.execute(sqlx)
        sqlx = "CREATE INDEX "  + ikp + "_javna_infrastruktura_geom_idx ON gis_podi."+ ikp + "_javna_infrastruktura USING GIST(geom)"
        crsx.execute(sqlx)
        sqlx = "SELECT Populate_Geometry_Columns('gis_podi."+ ikp + "_javna_infrastruktura'::regclass);"
        crsx.execute(sqlx)
        sqlx = "INSERT INTO gis_podi."+ ikp + "_javna_infrastruktura(geom) SELECT x.geom FROM privremena_javna_infrastruktura AS x;"
        crsx.execute(sqlx)
        e = zadnjaTocka.x()
        n = zadnjaTocka.y()
        sqlx = "DROP TABLE IF EXISTS privremeni_cilj;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE privremeni_cilj AS SELECT ST_GeomFromText('POINT(" + str(e) + " " + str(n) + ")', 3765) AS geom;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS gis_podi." + ikp + "_cilj;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE gis_podi." + ikp + "_cilj(gid serial primary key NOT NULL);"
        crsx.execute(sqlx)
        sqlx = "SELECT AddGeometryColumn ('gis_podi', '" + ikp + "_cilj', 'geom', 3765, 'POINT', 2, false);"
        crsx.execute(sqlx)
        sqlx = "CREATE INDEX "  + ikp + "_cilj_geom_idx ON gis_podi."+ ikp + "_cilj USING GIST(geom)"
        crsx.execute(sqlx)
        sqlx = "SELECT Populate_Geometry_Columns('gis_podi."+ ikp + "_cilj'::regclass);"
        crsx.execute(sqlx)
        sqlx = "INSERT INTO gis_podi."+ ikp + "_cilj(geom) SELECT x.geom FROM privremeni_cilj AS x;"
        crsx.execute(sqlx)
        sqlx = "COMMIT;"
        crsx.execute(sqlx)
        return
    
    def udaljenost(self, tA, tB):
        dE = tA.x() - tB.x()
        dN = tA.y() - tB.y()
        return math.sqrt( dE * dE + dN * dN )
    
    def popraviPocetak(self, qPT):
        crsx = self.cursor 
        e = qPT.x()
        n = qPT.y()
        sqlx = "DROP TABLE IF EXISTS pkp;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE pkp AS SELECT ST_GeomFromText('POINT(" + str(e) + " " + str(n) + ")', 3765) AS geom;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS popravak;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE popravak AS SELECT ST_ShortestLine(x.geom, y.geom) AS geom FROM pkp AS x, gis_podi.radna_parcela AS y;"
        crsx.execute(sqlx)
        sqlx = "SELECT ST_AsGeoJSON(x.geom) FROM popravak AS x;"
        crsx.execute(sqlx)
#         QgsMessageLog.logMessage(str(crsx.rowcount))
        for redak in crsx.fetchall():
            js = redak[0]
            g = ogr.CreateGeometryFromJson(js)
            q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
            tocke = self.extractPoints(q)
        e = tocke[1].x()
        n = tocke[1].y()
#         QgsMessageLog.logMessage(str(e) + "," + str(n))
        return QgsPointXY( e , n )
    
    def pronadjiTrazeniIndeksZaPrijelom(self, e, n, tocke):
        if len(tocke) < 3:
            indeks = 0
        else:
            indeks = -1
            dMIN = 100000000000000.0
            for j in range(0 , len(tocke) - 2):
                d = self.udaljenostTockeOdPolovista(e, n, tocke[j].x(), tocke[j].y(), tocke[j + 1].x(), tocke[j + 1].y())
#                 QgsMessageLog.logMessage(str(j) + ": " + str(d))
                if d < dMIN:
                    indeks = j 
                    dMIN = d
        return indeks 
    
    def udaljenostTockeOdPolovista(self, e0, n0, eA, nA, eB, nB):
        return self.udaljenost(QgsPoint(e0,n0), QgsPoint( (eA + eB) / 2.0, (nA + nB) / 2.0))
        
    def udaljenostTockeOdPravca(self, e0, n0, eA, nA, eB, nB):
        A = 0.0
        B = 0.0
        C = 0.0
        if math.fabs(eA - eB) < 0.0000001:
            A = 1.0
            B = 0.0
            C = -1.0 * eA
        else:
            k = (nA - nB) / (eA - eB)
            l = nA - k * eA
            A = k 
            B = -1.0
            C = l
        brojnik = math.fabs(A * e0 + B * n0 + C)
        nazivnik = math.sqrt(A * A + B * B)
        return (brojnik / nazivnik)
    
    def brisiRadneSlojeve(self):
        korijen = QgsProject.instance().layerTreeRoot()
        grupa = korijen.findGroup("PrivremeneParcele")
        for sloj in grupa.children():
            if "mreza_za_" in sloj.name():
                dmp = sloj.dump()
                id = dmp.split("=")[-1].strip()
                QgsProject.instance().removeMapLayer(id)
            elif "PrivremenaParcela" in sloj.name():
                platno = self.iface.mapCanvas()
                podrucje = sloj.layer().extent()
                platno.setExtent(podrucje)
                platno.zoomByFactor(math.sqrt(2.0))

    def jeLiSpojIdePreko(self, tPOC, tUPIK):
        odgovor = False
        crsx = self.cursor
        e1 = tPOC.x()
        e2 = tUPIK.x()
        n1 = tPOC.y()
        n2 = tUPIK.y()
        sqlx = "DROP TABLE IF EXISTS smanjene_natjecajne;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE smanjene_natjecajne AS SELECT x.gid AS gid, ST_Buffer(x.geom, -0.25) AS geom FROM gis_podi.natjecajne_cestice AS x;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS presjecene;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE presjecene AS SELECT * FROM smanjene_natjecajne AS x,"
        sqlx = sqlx + " ST_GeomFromText('LINESTRING(" + str(e1) + " " + str(n1) + "," + str(e2) + " " + str(n2) +")', 3765) "
        sqlx = sqlx + "AS y WHERE ST_Intersects(x.geom, y);"
        crsx.execute(sqlx)
        sqlx = "SELECT * FROM presjecene;"
        crsx.execute(sqlx)
        if crsx.rowcount == 0:
            odgovor = False
        else:
            odgovor = True
#         QgsMessageLog.logMessage("Odgovor >>" + str(odgovor))
        return odgovor

    def pgPrivatnaInfrastruktura(self, tocke, ikp, ppp, pu):
        crsx = self.cursor    
        znacajka = QgsFeature()
        znacajka.setGeometry(QgsGeometry.fromPolyline([ ppp , pu ]))
        self.precrtajSloj([znacajka], ikp + "_privatna_infrastruktura", "LineString?crs=EPSG:3765")
        tekst = "'LINESTRING("
        maxIndex = len(tocke) - 1
        indeks = -1
        for t in tocke:
            indeks = indeks + 1
            e = t.x()
            n = t.y()
            tekst = tekst + str(e) + " " + str(n)
            if indeks < maxIndex:
                tekst = tekst + ","
            else:
                tekst = tekst +")', 3765"
        sqlx = "DROP TABLE IF EXISTS privremena_privatna_infrastruktura;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE privremena_privatna_infrastruktura AS SELECT ST_GeomFromText(" + tekst + ") AS geom;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS gis_podi." + ikp + "_privatna_infrastruktura;"
        crsx.execute(sqlx)
        sqlx = "BEGIN;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE gis_podi." + ikp + "_privatna_infrastruktura(gid serial primary key NOT NULL);"
        crsx.execute(sqlx)
        sqlx = "SELECT AddGeometryColumn ('gis_podi', '" + ikp + "_privatna_infrastruktura', 'geom', 3765, 'LINESTRING', 2, false);"
        crsx.execute(sqlx)
        sqlx = "CREATE INDEX "  + ikp + "_privatna_infrastruktura_geom_idx ON gis_podi."+ ikp + "_privatna_infrastruktura USING GIST(geom)"
        crsx.execute(sqlx)
        sqlx = "SELECT Populate_Geometry_Columns('gis_podi."+ ikp + "_privatna_infrastruktura'::regclass);"
        crsx.execute(sqlx)
        sqlx = "INSERT INTO gis_podi."+ ikp + "_privatna_infrastruktura(geom) SELECT x.geom FROM privremena_privatna_infrastruktura AS x;"
        crsx.execute(sqlx)
        sqlx = "COMMIT;"
        crsx.execute(sqlx)
        sqlx = "SELECT ST_AsGeoJSON(c.geom) FROM gis_podi." + ikp + "_pocetak AS c;"
        crsx.execute(sqlx)
        znacajke = []
        for redak in crsx.fetchall():
            js = redak[0]
#                 QgsMessageLog.logMessage(js)
            g = ogr.CreateGeometryFromJson(js)
            q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
            z = QgsFeature()
            z.setGeometry(q)
            znacajke.append(z)
        self.precrtajSloj(znacajke, ikp + "_pocetak", "Point?crs=EPSG:3765")    
        return
    
    def pgPocetakPrikljucka(self, ppp, ikp):
        crsx = self.cursor
        e = ppp.x()
        n = ppp.y()
        sqlx = "DROP TABLE IF EXISTS privremeni_pocetak;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE privremeni_pocetak AS SELECT ST_GeomFromText('POINT(" + str(e) + " " + str(n) + ")', 3765) AS geom;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS gis_podi." + ikp + "_pocetak;"
        crsx.execute(sqlx)
        sqlx = "BEGIN;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE gis_podi." + ikp + "_pocetak(gid serial primary key NOT NULL);"
        crsx.execute(sqlx)
        sqlx = "SELECT AddGeometryColumn ('gis_podi', '" + ikp + "_pocetak', 'geom', 3765, 'POINT', 2, false);"
        crsx.execute(sqlx)
        sqlx = "CREATE INDEX "  + ikp + "_pocetak_geom_idx ON gis_podi."+ ikp + "_pocetak USING GIST(geom)"
        crsx.execute(sqlx)
        sqlx = "SELECT Populate_Geometry_Columns('gis_podi."+ ikp + "_pocetak'::regclass);"
        crsx.execute(sqlx)
        sqlx = "INSERT INTO gis_podi."+ ikp + "_pocetak(geom) SELECT x.geom FROM privremeni_pocetak AS x;"
        crsx.execute(sqlx)
        sqlx = "COMMIT;"
        crsx.execute(sqlx)    
        sqlx = "SELECT ST_AsGeoJSON(c.geom) FROM gis_podi." + ikp + "_pocetak AS c;"
        crsx.execute(sqlx)
        znacajke = []
        for redak in crsx.fetchall():
            js = redak[0]
#                 QgsMessageLog.logMessage(js)
            g = ogr.CreateGeometryFromJson(js)
            q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
            z = QgsFeature()
            z.setGeometry(q)
            znacajke.append(z)
        self.precrtajSloj(znacajke, ikp + "_pocetak", "Point?crs=EPSG:3765")
        return

    def pgUpiknuce(self, pu, ikp):
        crsx = self.cursor
        e = pu.x()
        n = pu.y()
        sqlx = "DROP TABLE IF EXISTS privremeno_upiknuce;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE privremeno_upiknuce AS SELECT ST_GeomFromText('POINT(" + str(e) + " " + str(n) + ")', 3765) AS geom;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS gis_podi." + ikp + "_upiknuce;"
        crsx.execute(sqlx)
        sqlx = "BEGIN;"
        crsx.execute(sqlx)
        sqlx = "CREATE TABLE gis_podi." + ikp + "_upiknuce(gid serial primary key NOT NULL);"
        crsx.execute(sqlx)
        sqlx = "SELECT AddGeometryColumn ('gis_podi', '" + ikp + "_upiknuce', 'geom', 3765, 'POINT', 2, false);"
        crsx.execute(sqlx)
        sqlx = "CREATE INDEX "  + ikp + "_upiknuce_geom_idx ON gis_podi."+ ikp + "_upiknuce USING GIST(geom)"
        crsx.execute(sqlx)
        sqlx = "SELECT Populate_Geometry_Columns('gis_podi."+ ikp + "_cilj'::regclass);"
        crsx.execute(sqlx)
        sqlx = "INSERT INTO gis_podi."+ ikp + "_upiknuce(geom) SELECT x.geom FROM privremeno_upiknuce AS x;"
        crsx.execute(sqlx)
        sqlx = "COMMIT;"
        crsx.execute(sqlx)    
        sqlx = "SELECT ST_AsGeoJSON(c.geom) FROM gis_podi." + ikp + "_upiknuce AS c;"
        crsx.execute(sqlx)
        znacajke = []
        for redak in crsx.fetchall():
            js = redak[0]
            g = ogr.CreateGeometryFromJson(js)
            q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
            z = QgsFeature()
            z.setGeometry(q)
            znacajke.append(z)
        self.precrtajSloj( znacajke, ikp + "_upiknuce", "Point?crs=EPSG:3765" )
        return e, n
           
    def pgOkna(self, ikp, iXXX, javnaMreza):
        crsx = self.cursor
        tt = ikp[iXXX] + "_tt"
        if iXXX == 0: #kanalizacija
            sqlx = "DROP TABLE IF EXISTS " + tt + ";"
            crsx.execute(sqlx)
            sqlx = "CREATE TEMPORARY TABLE " + tt + " AS SELECT a.geom AS geom FROM gis_podi.okno AS a WHERE a.namjena LIKE 'fekalna';"
            crsx.execute(sqlx)
        elif iXXX == 1: #vodovod
            sqlx = "DROP TABLE IF EXISTS " + tt + ";"
            crsx.execute(sqlx)
            sqlx = "CREATE TEMPORARY TABLE " + tt + " AS SELECT a.geom AS geom FROM gis_podi.okno AS a WHERE a.namjena LIKE 'vodovod';"
            crsx.execute(sqlx)
        elif iXXX == 2: #struja
            sqlx = "DROP TABLE IF EXISTS " + tt + ";"
            crsx.execute(sqlx) 
            sqlx = "CREATE TEMPORARY TABLE " + tt + " AS SELECT (ST_Dump(ST_Intersection(ST_Boundary(ST_CollectionExtract(ST_MakeValid(a.geom),3)), b.geom))).geom AS geom FROM gis_podi.trafostanica AS a, gis_podi.vod AS b;"
            crsx.execute(sqlx)
        elif iXXX == 3: #telefon
            sqlx = "DROP TABLE IF EXISTS " + tt + ";"
            crsx.execute(sqlx)
            sqlx = "CREATE TEMPORARY TABLE " + tt + " AS SELECT a.geom AS geom FROM gis_podi.zdenac AS a;"
            crsx.execute(sqlx)
        elif iXXX == 4: #uljna
            sqlx = "DROP TABLE IF EXISTS " + tt + ";"
            crsx.execute(sqlx)
            sqlx = "CREATE TEMPORARY TABLE " + tt + " AS SELECT a.geom AS geom FROM gis_podi.okno AS a WHERE a.namjena LIKE 'uljna';"
            crsx.execute(sqlx)
        tt2 = tt + "_2"
        sqlx = "DROP TABLE IF EXISTS okna;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE okna AS SELECT (ST_ShortestLine(a.geom, b.geom)) AS geom FROM " + tt + " AS a, " + javnaMreza + " AS b WHERE ST_Distance(a.geom, b.geom) < 1.0;"
        crsx.execute(sqlx)
        sqlx = "SELECT ST_AsGeoJSON(c.geom) FROM okna AS c;"
        crsx.execute(sqlx)
        okna = []
        znacajke = []
        for redak in crsx.fetchall():
            js = redak[0]
#                     QgsMessageLog.logMessage(js)
            g = ogr.CreateGeometryFromJson(js)
            q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
            tocke = self.extractPoints(q)
            ft = QgsFeature()
            ft.setGeometry(q)
            okna.append(QgsPointXY(tocke[1].x(), tocke[1].y()))
            znacajke.append(ft)
        return okna, znacajke
    
    def pgSegmentizacija(self, e, n, javnaMreza):
        crsx = self.cursor
        znacajke = []
        sqlx = "DROP TABLE IF EXISTS javna_mreza_segmentizirana;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE javna_mreza_segmentizirana(gid serial primary key NOT NULL)"
        crsx.execute(sqlx)
        sqlx = "SELECT AddGeometryColumn ('javna_mreza_segmentizirana', 'geom', 3765, 'LINESTRING', 2, false);"
        crsx.execute(sqlx)
        sqlx = "CREATE INDEX javna_mreza_segmentizirana_geom_idx ON javna_mreza_segmentizirana USING GIST(geom)"
        crsx.execute(sqlx)
        sqlx = "SELECT Populate_Geometry_Columns('javna_mreza_segmentizirana'::regclass);"
        crsx.execute(sqlx)
        sqlx = "INSERT INTO javna_mreza_segmentizirana(geom) SELECT (ST_Dump(x.geom)).geom AS geom FROM " + javnaMreza + " AS x;"
        crsx.execute(sqlx)
        sqlx = "INSERT INTO javna_mreza_segmentizirana(geom) SELECT x.geom AS geom FROM okna AS x;"
        crsx.execute(sqlx)
        sqlx = "SELECT * FROM (SELECT gid, ROW_NUMBER() OVER(PARTITION BY geom ORDER BY gid ASC) AS Row, geom FROM ONLY javna_mreza_segmentizirana) AS duple WHERE duple.Row > 1;"
        crsx.execute(sqlx)
#             QgsMessageLog.logMessage("Broj duplih >> " + str(crsx.rowcount))
        for redak in crsx.fetchall():
#                 QgsMessageLog.logMessage(str(redak[0]))
            sqlx = "DELETE FROM javna_mreza_segmentizirana WHERE gid = " + str(redak[0]) + ";"
            crsx.execute(sqlx)
        upik = "('POINT(" + str(e) + " " + str(n)+ ")',3765)"
        sqlx = "DROP TABLE IF EXISTS upik;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE upik AS SELECT ST_GeomFromText" + upik + "AS geom;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS najblizi_segment;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE najblizi_segment AS SELECT b.gid AS gid, b.geom AS geom FROM upik AS a, javna_mreza_segmentizirana AS b ORDER BY ST_Distance(a.geom, b.geom) ASC LIMIT 1;"
        crsx.execute(sqlx)
        sqlx = "SELECT ST_AsGeoJSON(x.geom), x.gid FROM najblizi_segment AS x;"
        crsx.execute(sqlx)
        sgID = ""
        tocke = []
        trazeniIndeks = -1
        for redak in crsx.fetchall():
            sgID = str(redak[1])
            js = redak[0]
            g = ogr.CreateGeometryFromJson(js)
            q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
            tocke = self.extractPoints(q)
#                 QgsMessageLog.logMessage(str(tocke))
            trazeniIndeks = self.pronadjiTrazeniIndeksZaPrijelom(e, n, tocke)
#                 QgsMessageLog.logMessage("Najbliza gid= " + sgID + ", indeks= " + str(trazeniIndeks))
        sqlx = "UPDATE javna_mreza_segmentizirana SET geom = ST_AddPoint(geom, "
        sqlx = sqlx + "ST_GeomFromText" + upik + ", " + str(trazeniIndeks + 1) + ") WHERE gid = "
        sqlx = sqlx + sgID + ";"
        crsx.execute(sqlx)
        sqlx = "SELECT ST_AsGeoJSON(c.geom) FROM javna_mreza_segmentizirana AS c;"
        crsx.execute(sqlx)
        for redak in crsx.fetchall():
            js = redak[0]
#                 QgsMessageLog.logMessage(js)
            g = ogr.CreateGeometryFromJson(js)
            q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
            z = QgsFeature()
            z.setGeometry(q)
            znacajke.append(z)
#             QgsMessageLog.logMessage("Prikljucak upiknuca na javnu mrezu...")
        self.precrtajSloj(znacajke, javnaMreza + "_okna", "LineString?crs=EPSG:3765")
        return

    def varijanteJavneInfrastrukture(self, javnaMreza, okna, pu):
        sloj = None
        for lyr in QgsProject.instance().mapLayers().values():
            if lyr.name() == (javnaMreza + "_okna"):
                sloj = lyr
                break
        i = 0
        trosak = 1000000000000000.0
        zapamcenePutanje = []
        zapamceniTroskovi = []
        indeks = - 1
        for okno in okna:
            i = i + 1
            if pu.distance(okno) < 0.01:
                    indeks = indeks + 1
                    zapamceniTroskovi.append(0.00)
                    putanja = []
                    putanja.append(pu)
                    putanja.append(okno)
                    zapamcenePutanje.append(putanja)
            elif pu.distance(okno) < 2500.0:
                director = QgsLineVectorLayerDirector( sloj, -1, '', '', '', 3)
                properter = QgsDistanceArcProperter()
                director.addProperter( properter )
                crs = sloj.crs()
                builder = QgsGraphBuilder(crs)
                pStart = pu
                pStop = okno
                tiedPoints = director.makeGraph(builder, [pStart, pStop])
                graph = builder.graph()
                tStart = tiedPoints[0]
                tStop = tiedPoints[1]
                idStart = graph.findVertex(tStart)
                idStop = graph.findVertex(tStop)
                (tree, cost) = QgsGraphAnalyzer.dijkstra(graph, idStart, 0)
                if tree[idStop] == -1:
                    pass
                else:
                    putanja = []
                    trosak = 0.0
                    dx = 0.0
                    dy = 0.0
                    curPos = idStop
                    while curPos != idStart:
                        putanja.append(graph.vertex(graph.arc(tree[curPos]).inVertex()).point())
                        curPos = graph.arc(tree[curPos]).outVertex();
                    putanja.append(tStart)
                    zapamcenePutanje.append(putanja)
                    predzadnja = None
                    for p in putanja:
                        if predzadnja is not None:
                            dx = p.x() - predzadnja.x() 
                            dy = p.y()- predzadnja.y()
                            trosak = trosak + math.sqrt( dx * dx + dy * dy )
                        predzadnja = p
                    indeks = indeks + 1
                    zapamceniTroskovi.append(trosak)
        return zapamcenePutanje, zapamceniTroskovi

    def infrastruktura(self):
        self.dockwidget.status.setText('Procesiranje... Pricekajte...')
        self.dockwidget.status.update()
        QgsMessageLog.logMessage("Stvaranje javnog dobra...")
        crsx = self.cursor
#         sqlx = "SELECT x.gid FROM gis_podi.natjecajne_cestice AS x, gis_podi.radna_parcela AS y WHERE ST_Equals(ST_SnapToGrid(x.geom, 0.0001),ST_SnapToGrid(y.geom, 0.0001));"
#         crsx.execute(sqlx)
#         gIDnatjecajne = ""
#         for redak in crsx.fetchall():
#             gIDnatjecajne = str(redak[0])
#         QgsMessageLog.logMessage("gID =" + gIDnatjecajne)
        lstKCBroj = [ '1306/106', '465/15', '4132/36', '4132/21', '4132/57' ]
        lstKO = [ 'DANILO BIRANJ', 'DONJE POLJE', 'DUBRAVA', 'DUBRAVA', 'DUBRAVA' ]
        lstIK = zip( lstKCBroj, lstKO)
        dioSQLa = "(VALUES "
        i = 0
        for ik in lstIK:
            i = i + 1
            dioSQLa = dioSQLa + "('" + ik[0] + "', '" + ik[1] + "'" 
            if i < len(lstIK):
                dioSQLa = dioSQLa + "), "
            else:
                dioSQLa = dioSQLa + ") "
        dioSQLa = dioSQLa + ") AS q(xkc,xko)"
        sqlx = "DROP TABLE IF EXISTS javno_dobro;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE javno_dobro AS SELECT * FROM gis_podi.katastarske_cestice AS p, "
        sqlx = sqlx + dioSQLa + " WHERE p.kc_broj = q.xkc AND p.ko = q.xko;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS pkc;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE pkc AS SELECT ST_Buffer(x.geom, 0.25) AS geom FROM gis_podi.radna_parcela AS x WHERE ST_Area(x.geom)>0.0625;"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS izlazne_fronte;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE izlazne_fronte AS SELECT (ST_Dump(ST_Intersection(x.geom, y.geom))).geom AS geom FROM pkc x INNER JOIN javno_dobro y ON ST_Intersects(x.geom,y.geom);"
        crsx.execute(sqlx)
        sqlx = "DROP TABLE IF EXISTS izlazne_fronte_JSON;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE izlazne_fronte_JSON AS SELECT ST_AsGeoJSON(x.geom,3) FROM izlazne_fronte x WHERE St_Area(x.geom)>0.0625;"
        crsx.execute(sqlx)
        sqlx ="SELECT * FROM izlazne_fronte_JSON;"
        crsx.execute(sqlx)
        if crsx.rowcount == 0:
            QgsMessageLog.logMessage("Parcela nema izlaznu frontu prema javnom dobru...")
            self.onClosePlugin()
            return
        znacajke=[]
        listaPoligona = []
        for javnaFronta in crsx.fetchall():
            listaTocaka = []
            jSONvar = javnaFronta[0]
            geom = ogr.CreateGeometryFromJson(jSONvar)
            qg = QgsGeometry.fromWkt(geom.ExportToIsoWkt())
            listaTocaka.append(self.extractPoints(qg))
            listaPoligona.append(listaTocaka)
            znacajka = QgsFeature()
            znacajka.setGeometry(qg)
            znacajke.append(znacajka)
        gFronta = QgsGeometry.fromMultiPolygon([[[QgsPoint(point[0], point[1]) for point in polygon] for polygon in listaTocaka]])
        gFronta = gFronta.buffer( 0.1, 1)
        sqlx = "DROP TABLE IF EXISTS intracentroid;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE intracentroid AS SELECT ST_AsGeoJSON(ST_PointOnSurface(x.geom),3) FROM gis_podi.radna_parcela AS x;"
        crsx.execute(sqlx)
        sqlx = "SELECT * FROM intracentroid"
        crsx.execute(sqlx)
        intraCentroid = None
        for cestica in crsx.fetchall():
            jSONvar = cestica[0]
            geom = ogr.CreateGeometryFromJson(jSONvar)
            intraCentroid = QgsGeometry.fromWkt(geom.ExportToIsoWkt())
        parcela = None
        sqlx = "SELECT ST_AsGeoJSON(x.geom, 3) FROM gis_podi.radna_parcela AS x;"
        crsx.execute(sqlx)
        for cestica in crsx.fetchall():
            jSONvar = cestica[0]
            geom = ogr.CreateGeometryFromJson(jSONvar)
            parcela = QgsGeometry.fromWkt(geom.ExportToIsoWkt())
        QgsMessageLog.logMessage("Trazenje izlazne fronte...")
        sqlx = "DROP TABLE IF EXISTS pkx;"
        crsx.execute(sqlx)
        sqlx = "CREATE TEMPORARY TABLE pkx AS SELECT ST_Intersection(x.geom, y.geom) AS geom FROM izlazne_fronte AS x, gis_podi.radna_parcela AS y;"
        crsx.execute(sqlx)
        
        listaImenaVodova = [ 'kanalizacijska_cijev' , 'vodovodna_cijev' , 'vod', 'trasa', 'kanalizacijska_cijev']
#         listaImenaVodova = [ 'kanalizacijska_cijev' ]
        imeKaoPrefiks = ['kanalizacija', 'voda', 'struja', 'telefon' , 'deoleinizacija']
        polozaj_pocetka_prikljucka = [ QgsPoint(), QgsPoint(), QgsPoint(), QgsPoint(), QgsPoint()]
        polozaj_upiknuca = [ QgsPoint(), QgsPoint(), QgsPoint(), QgsPoint(), QgsPoint()]
        iXXX = -1
        for imeVoda in listaImenaVodova:
#             self.dockwidget.status.setText('Izrada grafa za: ' + str(imeKaoPrefiks[iXXX]) )
#             self.dockwidget.status.repaint()
            iXXX = iXXX + 1
#             QgsMessageLog.logMessage( str ( iXXX + 1 ) + "  " + imeKaoPrefiks[iXXX] + ":" )
            komadVoda = imeKaoPrefiks[iXXX] + "_na_parceli"
            pocetakVoda = "pocetak_za_" + imeKaoPrefiks[iXXX]
            imeVodaSPrefiksom = "gis_podi."+ imeVoda
            javnaMreza = "mreza_za_" + imeKaoPrefiks[iXXX]
#             QgsMessageLog.logMessage(str(iXXX+1) + ": " + imeVoda)
            sqlx = "DROP TABLE IF EXISTS " + javnaMreza + "_gc;"
            crsx.execute(sqlx)
            self.nulirajSloj(imeKaoPrefiks[iXXX] + "_privatna_infrastruktura", "LineString?crs=EPSG:3765")
            self.nulirajSloj(imeKaoPrefiks[iXXX] + "_javna_infrastruktura", "LineString?crs=EPSG:3765")
            if iXXX == 0 or iXXX == 4:
                if iXXX == 0:
                    sqlx = "DROP TABLE IF EXISTS samo_fekalna;"
                    crsx.execute(sqlx)
                    sqlx = "CREATE TEMPORARY TABLE samo_fekalna AS SELECT * FROM " + imeVodaSPrefiksom + " WHERE tip LIKE 'fekalna';"
                    crsx.execute(sqlx) 
                    sqlx = "CREATE TEMPORARY TABLE " + javnaMreza + "_gc AS SELECT (ST_Union(ST_Intersection(u1.geom, u2.geom))) AS geom FROM samo_fekalna AS u1 INNER JOIN javno_dobro u2 ON ST_INTERSECTS(u1.geom, u2.geom);"
                else:
                    sqlx = "DROP TABLE IF EXISTS samo_uljna;"
                    crsx.execute(sqlx)
                    sqlx = "CREATE TEMPORARY TABLE samo_uljna AS SELECT * FROM " + imeVodaSPrefiksom + " WHERE tip LIKE 'uljna';"
                    crsx.execute(sqlx) 
                    sqlx = "CREATE TEMPORARY TABLE " + javnaMreza + "_gc AS SELECT (ST_Union(ST_Intersection(u1.geom, u2.geom))) AS geom FROM samo_uljna AS u1 INNER JOIN javno_dobro u2 ON ST_INTERSECTS(u1.geom, u2.geom);"
            else:
                if iXXX ==1:
                    sqlx = "DROP TABLE IF EXISTS samo_voda;"
                    crsx.execute(sqlx)
                    sqlx = "CREATE TEMPORARY TABLE samo_voda AS SELECT * FROM " + imeVodaSPrefiksom + " WHERE CAST(dn AS double precision) < 299.0;"
                    crsx.execute(sqlx) 
                    sqlx = "CREATE TEMPORARY TABLE " + javnaMreza + "_gc AS SELECT (ST_Union(ST_Intersection(u1.geom, u2.geom))) AS geom FROM samo_voda AS u1 INNER JOIN javno_dobro u2 ON ST_INTERSECTS(u1.geom, u2.geom);"
#                     sqlx = "CREATE TEMPORARY TABLE " + javnaMreza + "_gc AS SELECT ST_Union(u1.geom) AS geom FROM samo_voda AS u1;"
                else:
                    sqlx = "CREATE TEMPORARY TABLE " + javnaMreza + "_gc AS SELECT ST_Union(u1.geom) AS geom FROM " + imeVodaSPrefiksom + " AS u1;"
            crsx.execute(sqlx)
            # treba ispitati da li je javnaMreza geom.kolekcija/ekstraktirati i dodati prostorni indeks
            sqlx = "DROP TABLE IF EXISTS "+ javnaMreza + ";"
            crsx.execute(sqlx)
            sqlx = "CREATE TEMPORARY TABLE " + javnaMreza + " AS SELECT ST_CollectionExtract(x.geom, 2) AS geom FROM " + javnaMreza + "_gc AS x;"
            crsx.execute(sqlx)
#             QgsMessageLog.logMessage("Stvaranje javne mreze...")
            #
            # 9.
            # a) da li se javnaMreza i izlazna_fronta sijeku
            # DA >> pocetak je tocka unutar cestice
            # NE >> upiknuce je tocka na rubu
            # ako ima vise presjeka odabire se:
            # prvo onaj koji izlazi na javno_dobro
            # drugo onaj koji je kraci
            #
            # b) ako se javnaMreza i predmetna_parcela ne sijeku >> trazi se najkraci spoj
            #
            sqlx = "DROP TABLE IF EXISTS " + komadVoda + ";"
            crsx.execute(sqlx)
            sqlx = "CREATE TEMPORARY TABLE " + komadVoda + " AS SELECT (ST_Dump(ST_Intersection(a1.geom, a2.geom))).geom AS geom FROM " + javnaMreza + " AS a1 INNER JOIN izlazne_fronte AS a2 ON ST_Intersects(a1.geom, a2.geom);"
            crsx.execute(sqlx)
            sqlx ="SELECT ST_AsGeoJSON(b.geom) FROM " + komadVoda + " AS b;"
            crsx.execute(sqlx)
            znacajke = []
            tocke = []
            privatnaOK = False
            polozaj_pocetka_prikljucka[iXXX] = None
            polozaj_upiknuca[iXXX] = None
            if crsx.rowcount == 0:
                # nema presjeka treba traziti najblizu (slucaj b) izlaznoj fronti)
#                 QgsMessageLog.logMessage("nema presjeka")
                sqlx = "DROP TABLE IF EXISTS jm_dampana;"
                crsx.execute(sqlx)
                sqlx = "CREATE TEMPORARY TABLE jm_dampana AS SELECT (ST_Dump(x.geom)).geom As geom FROM " + javnaMreza + " AS x;"
                crsx.execute(sqlx)
                sqlx = "DROP TABLE IF EXISTS " + pocetakVoda + ";"
                crsx.execute(sqlx)
                sqlx = "CREATE TEMPORARY TABLE " + pocetakVoda + " AS SELECT ST_ShortestLine(a.geom, b.geom) AS geom FROM jm_dampana AS a, izlazne_fronte AS b;"
                crsx.execute(sqlx)
                sqlx = "SELECT ST_AsGeoJSON(c.geom, 3) FROM " + pocetakVoda + " AS c;"
                crsx.execute(sqlx)
                udaljenost = 10000000000000000.0
                znacajka = QgsFeature()
#                 QgsMessageLog.logMessage("Slucaj B: " + str(crsx.rowcount))
                for redak in crsx.fetchall():
                    js = redak[0]
                    g = ogr.CreateGeometryFromJson(js)
                    q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
                    tocke = self.extractPoints(q)
                    q1 = QgsGeometry.fromPointXY(QgsPointXY(tocke[0].x(), tocke[0].y()))
                    q2 = QgsGeometry.fromPointXY(QgsPointXY(tocke[1].x(), tocke[1].y()))
                    pt1 = QgsPointXY(tocke[0].x(), tocke[0].y())
                    pt2 = QgsPointXY(tocke[1].x(), tocke[1].y())
                    d = QgsGeometry.distance( q1, q2 )
#                     QgsMessageLog.logMessage("JM: " + str(tocke[0][0]) + "," + str(tocke[0][1]))
#                     QgsMessageLog.logMessage("IF: " + str(tocke[1][0]) + "," + str(tocke[1][1]))
                    if d < udaljenost and not(self.jeLiSpojIdePreko(pt2, pt1)):
#                         QgsMessageLog.logMessage("d= " + str(d))
                        udaljenost = d
#                         znacajka.setGeometry(q)
                        privatnaOK = True
                        polozaj_pocetka_prikljucka[iXXX] = pt2
                        polozaj_upiknuca[iXXX] = pt1
#                         polozaj_pocetka_prikljucka[iXXX] = self.popraviPocetak(polozaj_pocetka_prikljucka[iXXX])
#                 QgsMessageLog.logMessage("Pocetak:" + str(polozaj_pocetka_prikljucka[iXXX].x()) + ";" + str(polozaj_pocetka_prikljucka[iXXX].y())) 
#                 QgsMessageLog.logMessage("Upiknuce:" + str(polozaj_upiknuca[iXXX].x()) + ";" + str(polozaj_upiknuca[iXXX].y())) 
            else:
                # ima presjeka (slucaj a)
#                 QgsMessageLog.logMessage("ima presjeka")
                sqlx = "DROP TABLE IF EXISTS " + pocetakVoda + ";"
                crsx.execute(sqlx)
                sqlx = "CREATE TEMPORARY TABLE " + pocetakVoda + " AS SELECT ST_ShortestLine(a.geom, b.geom) AS geom FROM " + komadVoda + " AS a, pkc AS b;"
                crsx.execute(sqlx)
                sqlx = "SELECT ST_AsGeoJSON(c.geom, 3) FROM " + pocetakVoda + " AS c;"
                crsx.execute(sqlx)
                #QgsMessageLog.logMessage("Broj presjeka:" + str(crsx.rowcount))
#                 QgsMessageLog.logMessage("Slucaj A: " + str(crsx.rowcount))
                udaljenost = 10000000000000000.0
                znacajka = QgsFeature()
#                 QgsMessageLog.logMessage("Slucaj A: " + str(crsx.rowcount))
                for redak in crsx.fetchall():
                    js = redak[0]
                    g = ogr.CreateGeometryFromJson(js)
                    q = QgsGeometry.fromWkt(g.ExportToIsoWkt())
                    tocke = self.extractPoints(q)
                    q1 = QgsGeometry.fromPointXY(QgsPointXY(tocke[0].x(), tocke[0].y()))
                    q2 = QgsGeometry.fromPointXY(QgsPoint(tocke[1].x(), tocke[1].y()))
                    d = QgsGeometry.distance(q1, q2)
                    pt1 = QgsPointXY(tocke[0].x(), tocke[0].y())
                    pt2 = QgsPointXY(tocke[1].x(), tocke[1].y())
                    if d < udaljenost and not(self.jeLiSpojIdePreko(pt2, pt1)):
                        udaljenost = d
                        znacajka.setGeometry(q)
                        polozaj_upiknuca[iXXX] = pt1
                        polozaj_pocetka_prikljucka[iXXX] = pt2
                        privatnaOK = True
#                 QgsMessageLog.logMessage("Pocetak:" + str(polozaj_pocetka_prikljucka[iXXX].x()) + ";" + str(polozaj_pocetka_prikljucka[iXXX].y())) 
#                 QgsMessageLog.logMessage("Upiknuce:" + str(polozaj_upiknuca[iXXX].x()) + ";" + str(polozaj_upiknuca[iXXX].y())) 
            if privatnaOK:
                if self.udaljenost(polozaj_upiknuca[iXXX], polozaj_pocetka_prikljucka[iXXX]) < 0.001:
                    polozaj_pocetka_prikljucka[iXXX] = self.popraviPocetak(polozaj_pocetka_prikljucka[iXXX])
                    polozaj_upiknuca[iXXX] = polozaj_pocetka_prikljucka[iXXX]
                else:
                    polozaj_pocetka_prikljucka[iXXX] = self.popraviPocetak(polozaj_pocetka_prikljucka[iXXX])
                self.pgPrivatnaInfrastruktura(tocke, imeKaoPrefiks[iXXX], polozaj_pocetka_prikljucka[iXXX], polozaj_upiknuca[iXXX])
                self.pgPocetakPrikljucka(polozaj_pocetka_prikljucka[iXXX], imeKaoPrefiks[iXXX])
                e, n = self.pgUpiknuce(polozaj_upiknuca[iXXX], imeKaoPrefiks[iXXX])
                # 10. objedinjavanje mrea i tokastih topologija (tt)
                #  - kanalizacijska_mreza + okna kojemu je namjena fekalna
                #  - vodovodna_mreza + okna koje kojemu je namjena vodovod
                #  - ee_mreza + trafostanice
                #  - tk_mreza + zdenci
                #  - kanalizacijska_mreza + okna kojemu je namjena uljna
                #
                okna, znacajke = self.pgOkna(imeKaoPrefiks, iXXX, javnaMreza)
                self.pgSegmentizacija(e,n, javnaMreza)
                self.privat[iXXX] = self.udaljenost(polozaj_pocetka_prikljucka[iXXX], polozaj_upiknuca[iXXX])
                # Dijkstra
                zapamcenePutanje, zapamceniTroskovi = self.varijanteJavneInfrastrukture(javnaMreza, okna, polozaj_upiknuca[iXXX])
                QgsMessageLog.logMessage(imeKaoPrefiks[iXXX].title() + "/Broj rjesenja: " + str(len(zapamcenePutanje)))
                if len(zapamcenePutanje) > 0:
                    rjecnik = zip(zapamceniTroskovi, zapamcenePutanje)
                    sortirano = sorted(rjecnik, key=lambda duz: duz[0])
                    self.troskovi[iXXX] = []
                    self.putanje[iXXX] = []
                    for sduz,sgeom in sortirano:
                        self.troskovi[iXXX].append("{0:.2f}".format(sduz))
                        self.putanje[iXXX].append(sgeom)
                    if iXXX == 0:
                        self.dockwidget.cboKANALIZACIJA.clear()
                        self.dockwidget.cboKANALIZACIJA.addItems(self.troskovi[iXXX])
                        self.kanalizacija = self.putanje[iXXX][0]
                    elif iXXX == 1:
                        self.dockwidget.cboVODOVOD.clear()
                        self.dockwidget.cboVODOVOD.addItems(self.troskovi[iXXX])
                        self.vodovod = self.putanje[iXXX][0]
                    elif iXXX == 2:
                        self.dockwidget.cboSTRUJA.clear()
                        self.dockwidget.cboSTRUJA.addItems(self.troskovi[iXXX])
                        self.struja = self.putanje[iXXX][0]
                    elif iXXX == 3:
                        self.dockwidget.cboTELEFON.clear()
                        self.dockwidget.cboTELEFON.addItems(self.troskovi[iXXX])
                        self.telefon = self.putanje[iXXX][0]
                    elif iXXX == 4:
                        self.dockwidget.cboDEOLEINIZACIJA.clear()
                        self.dockwidget.cboDEOLEINIZACIJA.addItems(self.troskovi[iXXX])
                        self.deoleinizacija = self.putanje[iXXX][0]
                    tocke = self.extractPoints(QgsGeometry.fromPolyline(self.putanje[iXXX][0]))
                    zadnjaTocka = tocke[len(tocke) - 1]
                    e = zadnjaTocka.x()
                    n = zadnjaTocka.y()
                    sqlx = "DROP TABLE IF EXISTS privremeni_cilj;"
                    crsx.execute(sqlx)
                    sqlx = "CREATE TABLE privremeni_cilj AS SELECT ST_GeomFromText('POINT(" + str(e) + " " + str(n) + ")', 3765) AS geom;"
                    crsx.execute(sqlx)
                    self.ciljevi[iXXX] = QgsPointXY(e,n) 
                    sqlx = "DROP TABLE IF EXISTS gis_podi." + imeKaoPrefiks[iXXX] + "_cilj;"
                    crsx.execute(sqlx)
                    sqlx = "BEGIN;"
                    crsx.execute(sqlx)
                    sqlx = "CREATE TABLE gis_podi." + imeKaoPrefiks[iXXX] + "_cilj(gid serial primary key NOT NULL);"
                    crsx.execute(sqlx)
                    sqlx = "SELECT AddGeometryColumn ('gis_podi', '" + imeKaoPrefiks[iXXX] + "_cilj', 'geom', 3765, 'POINT', 2, false);"
                    crsx.execute(sqlx)
                    sqlx = "CREATE INDEX "  + imeKaoPrefiks[iXXX] + "_cilj_geom_idx ON gis_podi."+ imeKaoPrefiks[iXXX] + "_cilj USING GIST(geom)"
                    crsx.execute(sqlx)
                    sqlx = "SELECT Populate_Geometry_Columns('gis_podi."+ imeKaoPrefiks[iXXX] + "_cilj'::regclass);"
                    crsx.execute(sqlx)
                    sqlx = "INSERT INTO gis_podi."+ imeKaoPrefiks[iXXX] + "_cilj(geom) SELECT x.geom FROM privremeni_cilj AS x;"
                    crsx.execute(sqlx)
                    sqlx = "COMMIT;"
                    crsx.execute(sqlx)
                else:
                    self.ciljevi[iXXX] = None
                    QgsMessageLog.logMessage(imeKaoPrefiks[iXXX].title() + "/Nema javne infrastrukture!")
            else:
                self.ciljevi[iXXX] = None
                QgsMessageLog.logMessage(imeKaoPrefiks[iXXX].title() + "/Nema privatne infrastrukture!")
        self.dockwidget.status.setText('Gotovo !')
        self.dockwidget.status.repaint()
        self.dockwidget.gumb.setEnabled(True)
        return

class Literal(str):
    
    def __conform__(self, quote):
        return self
    @classmethod
    def mro(cls):
        return (object, )
    
    def getquoted(self):
        return str(self)
